#include <QTRSensors.h>

#define Kp 0.3                  // experiment to determine this, start by something small that just makes your bot follow the line at a slow speed
#define Kd 1                    // experiment to determine this, slowly increase the speeds and adjust this value. (Note: Kp < Kd) 
#define RIGHT_MAX_SPEED 255     // max speed of the robot
#define LEFT_MAX_SPEED 255      // max speed of the robot
#define RIGHT_BASE_SPEED 100    // this is the speed at which the motors should spin when the robot is perfectly on the line
#define LEFT_BASE_SPEED 100     // this is the speed at which the motors should spin when the robot is perfectly on the line
#define NUM_SENSORS 5           // number of sensors used
#define TIMEOUT 2500            // waits for 2500 us for sensor outputs to go low
#define EMITTER_PIN 2           // emitter is controlled by digital pin 2

#define leftMotor1 4
#define leftMotor2 5
#define leftMotorPWM 9
#define rightMotor1 6 
#define rightMotor2 7
#define rightMotorPWM 10
//#define motorPower 8

// sensor connected through analog pins A0 - A5 i.e. digital pins 14-19
QTRSensorsRC qtrrc((unsigned char[]) {14, 15, 16, 17, 18}, NUM_SENSORS, TIMEOUT, EMITTER_PIN); 
unsigned int sensorValues[NUM_SENSORS];
int lastError = 0;
int error = 0;
int position;
int motorSpeed;
int leftMotorSpeed;
int rightMotorSpeed;


void setup()
{
  pinMode(rightMotor1, OUTPUT);
  pinMode(rightMotor2, OUTPUT);
  pinMode(rightMotorPWM, OUTPUT);
  pinMode(leftMotor1, OUTPUT);
  pinMode(leftMotor2, OUTPUT);
  pinMode(leftMotorPWM, OUTPUT);
//  pinMode(motorPower, OUTPUT);
  
  int i;
  for (int i = 0; i < 100; i++) {
    // calibrate for sometime by sliding the sensors across the line, or you may use auto-calibration instead

    // comment this part out for automatic calibration 
//  if ( i  < 25 || i >= 75 ) // turn to the left and right to expose the sensors to the brightest and darkest readings that may be encountered
//     turn_right();  
//   else
//     turn_left();
    qtrrc.calibrate();   
    delay(100);
//  delay(2000); // wait for 2s to position the bot before entering the main loop 
     
  // comment out for serial printing
    Serial.begin(9600);
    for (int i = 0; i < NUM_SENSORS; i++)
    {
      Serial.print(qtrrc.calibratedMinimumOn[i]);
      Serial.print(' ');
    }
    Serial.println();
    
    for (int i = 0; i < NUM_SENSORS; i++)
    {
      Serial.print(qtrrc.calibratedMaximumOn[i]);
      Serial.print(' ');
    }
//  Serial.println();
    Serial.println();
  }
} 


void loop()
{
  // get calibrated readings along with the line position, refer to the QTR Sensors Arduino Library for more details on line position
  position = qtrrc.readLine(sensorValues);
  
  // print the sensor values as numbers from 0 to 1000, where 0 means maximum reflectance and
  // 1000 means minimum reflectance, followed by the line position
  for (unsigned char i = 0; i < NUM_SENSORS; i++)
  {
    Serial.print(sensorValues[i]);
    Serial.print('\t');
  }
  // Serial.println(); // uncomment this line if you are using raw values
  Serial.print(position); // comment this line out if you are using raw values
  Serial.print('\t');
  
  delay(250);
  
  error = position - 2000;

  motorSpeed = Kp * error + Kd * (error - lastError);
  Serial.println(motorSpeed);

  lastError = error;

  rightMotorSpeed = RIGHT_BASE_SPEED + motorSpeed;
  leftMotorSpeed = LEFT_BASE_SPEED - motorSpeed;
  
  if (rightMotorSpeed > RIGHT_MAX_SPEED ) rightMotorSpeed = RIGHT_MAX_SPEED; // prevent the motor from going beyond max speed
  if (leftMotorSpeed > LEFT_MAX_SPEED ) leftMotorSpeed = LEFT_MAX_SPEED; // prevent the motor from going beyond max speed
  if (rightMotorSpeed < 0) rightMotorSpeed = 0; // keep the motor speed positive
  if (leftMotorSpeed < 0) leftMotorSpeed = 0; // keep the motor speed positive
  
//  digitalWrite(motorPower, HIGH); // move forward with appropriate speeds
  digitalWrite(rightMotor1, HIGH);
  digitalWrite(rightMotor2, LOW);
  analogWrite(rightMotorPWM, rightMotorSpeed);
//  digitalWrite(motorPower, HIGH);
  digitalWrite(leftMotor1, HIGH);
  digitalWrite(leftMotor2, LOW);
  analogWrite(leftMotorPWM, leftMotorSpeed);
}




